/*
 * Copyright (c) 2018 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.1
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0

Item {
    
    property string amount: '0'
    property double amountDefault: 0.200
    property bool blockUpdate: true
	property real maxFilter: 1000
    property real maxUser: 1000
    property var defaultParameters: [amount]
    property double startValue: 0.0
    property double middleValue: 0.5
    property double endValue: 0.0
   
    width: 250
    height: 100
    
    Component.onCompleted: {
        if (filter.isNew) {
            filter.set(amount, amountDefault)   
            filter.savePreset(preset.defaultParameters)
        } else {
            middleValue = filter.getDouble(amount, filter.animateIn)
            if (filter.animateIn > 0)
                startValue = filter.getDouble(amount, 0)
            if (filter.animateOut > 0)
                endValue = filter.getDouble(amount, filter.duration - 1)
        }
        setControls()
    }

    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }

    function setControls() {
        var position = getPosition()
        blockUpdate = true
        amountSlider.value = filter.getDouble(amount, position) * maxFilter
        blockUpdate = false
        amountSlider.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
    }

    function updateFilter(position) {
        if (blockUpdate) return
        var value = amountSlider.value / maxFilter

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0)
                startValue = value
            else if (position >= filter.duration - 1 && filter.animateOut > 0)
                endValue = value
            else
                middleValue = value
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty(amount)
            amountKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set(amount, startValue, 0)
                filter.set(amount, middleValue, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set(amount, middleValue, filter.duration - filter.animateOut)
                filter.set(amount, endValue, filter.duration - 1)
            }
        } else if (!amountKeyframesButton.checked) {
            filter.resetProperty(amount)
            filter.set(amount, middleValue)
        } else if (position !== null) {
            filter.set(amount, value, position)
        }
    }

    GridLayout {
        anchors.fill: parent
        anchors.margins: 8
        columns: 4

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }
        Preset {
            id: preset
			Layout.columnSpan: 3
            parameters: defaultParameters
            onBeforePresetLoaded: filter.resetProperty(amount)
            onPresetSelected: {
                setControls()
                amountKeyframesButton.checked = filter.keyframeCount(amount) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                middleValue = filter.getDouble(amount, filter.animateIn)
                if (filter.animateIn > 0)
                    startValue = filter.getDouble(amount, 0)
                if (filter.animateOut > 0)
                    endValue = filter.getDouble(amount, filter.duration - 1)
            }
        }

        Label {
            text: qsTr('Amount')
            Layout.alignment: Qt.AlignRight
        }
        SliderSpinner {
            id: amountSlider
            minimumValue: 0
            maximumValue: maxUser
			stepSize: 1
            decimals: 0
			onValueChanged: updateFilter(getPosition())
        }
        UndoButton {
            onClicked: amountSlider.value = amountDefault * maxFilter
        }
        KeyframesButton {
            id: amountKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(amount) > 0
            onToggled: {
                var value = amountSlider.value / maxFilter
                if (checked) {
                    blockUpdate = true
                    filter.clearSimpleAnimation(amount)
                    blockUpdate = false
                    filter.set(amount, value, getPosition())
                } else {
                    filter.resetProperty(amount)
                    filter.set(amount, value)
                }
            }
        }
  
        Item {
            Layout.fillHeight: true;
        }
    }

    Connections {
        target: filter
        onInChanged: updateFilter(null)
        onOutChanged: updateFilter(null)
        onAnimateInChanged: updateFilter(null)
        onAnimateOutChanged: updateFilter(null)
    }

    Connections {
        target: producer
        onPositionChanged: setControls()
    }
}